﻿#region usings
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;

using IdeaBlade.EntityModel;
using System;
using IdeaBlade.EntityModel.Security;


#endregion

namespace FirstSilverlightApp {

  public partial class Page : UserControl {

    #region ctor & load

    public Page() {
      InitializeComponent();
      Loaded += Page_Loaded;
    }

    void Page_Loaded(object sender, RoutedEventArgs e) {
      CreateEntityManager(); // the gateway to persistence
      InitializeQueries();
      Reset();
    }

    private void CreateEntityManager() {
      WriteMessage("Creating EntityManager ...");
      _entityManager = new NorthwindIBEntities(false);
    }


    #endregion

    #region Button Click Handlers

    private void btnConnect_Click(object sender, RoutedEventArgs e) {
      Connect(); // to the server
    }

    private void btnLogin_Click(object sender, RoutedEventArgs e) {
      Login(); // for security and identification
    }

    private void btnFetch_Click(object sender, RoutedEventArgs e) {
      Fetch(); // Get from the database
    }

    private void btnSave_Click(object sender, RoutedEventArgs e) {
      Save();
    }

    private void btnLogout_Click(object sender, RoutedEventArgs e) {
      Logout();
    }

    private void btnReset_Click(object sender, RoutedEventArgs e) {
      Reset();
    }
    #endregion

    #region Initializequeries

    private void InitializeQueries() {
      AddQueries();
      InitializeQueryCombo();
    }

    private void InitializeQueryCombo() {
      var qNames = _queries.Keys.ToList();
      queryCombo.ItemsSource = qNames;
      queryCombo.SelectedItem = qNames.FirstOrDefault();
      queryCombo.SelectionChanged += delegate { Fetch(); };
    }

    #endregion

    #region Fetch

    private async void Fetch() {

      var query = GetQuery();
      if (null == query) {
        WriteMessage("No query selected; pick a query.");
        return;
      }

      WriteMessage("Fetching ...");

      try {
        var results = await _entityManager.ExecuteQueryAsync(query);
        dg.ItemsSource = results;
        ReportFetchCount(results);
      } catch (Exception e) {
        WriteMessage(e.Message);
      }
    }

    private IEntityQuery GetQuery() {
      var queryName = (string)queryCombo.SelectedItem;
      return queryName == null ? null : _queries[queryName];
    }

    private void ReportFetchCount(IEnumerable result) {
      WriteMessage(string.Format("Retrieved {0} customers", ((ICollection)result).Count));
    }

    #endregion

    #region Queries

    private void AddQueries() {

      _queries = new Dictionary<string, IEntityQuery>();

      _queries.Add("Get all Customers", _entityManager.Customers);

      _queries.Add("Get Customers starting with A",
                  _entityManager.Customers
                    .Where(c => c.CompanyName.StartsWith("A")));

      _queries.Add("Get Customers with Orders in Oregon",
                  _entityManager.Customers
                    .Where(c => c.Orders.Any(o => o.ShipRegion == "OR")));

      _queries.Add("Get Customers starting with B from cache-only",
                  _entityManager.Customers
                    .Where(c => c.CompanyName.StartsWith("B"))
                    .With(QueryStrategy.CacheOnly));
    }

    #endregion

    #region Reset

    private void Reset() {

      // Start all over again ...

      btnLogin.IsEnabled = false;
      btnFetch.IsEnabled = false;
      btnLogout.IsEnabled = false;
      btnSave.IsEnabled = false;
      queryCombo.IsEnabled = false;

      dg.ItemsSource = null;

      _entityManager.Clear();
      if (_entityManager.IsLoggedIn) Logout();
      if (_entityManager.IsConnected) _entityManager.Disconnect();

      WriteMessage("Ready to connect ...");
    }

    #endregion

    #region Connect

    private async void Connect() {
      if (!_entityManager.IsConnected) WriteMessage("Connecting ...");
      try {
        await _entityManager.ConnectAsync();
        WriteMessage("Connected");
        btnLogin.IsEnabled = true;
      } catch (Exception e) {
        WriteMessage(e.Message);
      }
    }

    #endregion

    #region Login

    private async void Login() {
      // Use will be logged in as a guest.
      LoginCredential cred = null;

      WriteMessage("Logging in ...");

      try {
        _entityManager.AuthenticationContext = await Authenticator.Instance.LoginAsync(cred);
        WriteMessage("Logged in '" + _entityManager.AuthenticationContext.Principal.Identity.Name + "'");
        queryCombo.IsEnabled = true;
        btnFetch.IsEnabled = true;
        btnSave.IsEnabled = true;
        btnLogout.IsEnabled = true;
      } catch (Exception e) {
        WriteMessage(e.Message);
      }
    }

    #endregion

    #region Logout

    private async void Logout() {
      WriteMessage("Logging out ...");

      await Authenticator.Instance.LogoutAsync(_entityManager.AuthenticationContext);
      WriteMessage("Logged out");
      queryCombo.IsEnabled = false;
      btnFetch.IsEnabled = false;
      btnSave.IsEnabled = false;
      btnLogin.IsEnabled = true;
      btnLogout.IsEnabled = false;
    }

    #endregion

    #region Save

    private async void Save() {

      if (!_entityManager.HasChanges()) {
        WriteMessage("No changes to save.");
        return;
      }

      WriteMessage("Saving ...");

      try {
        await _entityManager.SaveChangesAsync();
        WriteMessage("Changes saved");
      } catch (Exception e) {
        WriteMessage(e.Message);
      }
    }

    #endregion

    #region Misc

    private void WriteMessage(string msg) {
      txtStatus.Text = msg;
    }

    #endregion

    #region Fields

    private NorthwindIBEntities _entityManager;
    private Dictionary<string, IEntityQuery> _queries;

    #endregion
  }
}
